##### DEMONSTRATIONS FOR RESEARCH NOTE #########
#load in cumulative GSS cross-section
library(foreign)
d=read.dta("~/Documents/Duke/Predictions with MI data/GSS_status.dta") ## REPLACE THIS WITH PATHNAME TO FILE ON YOUR COMPUTER, AND ALTER IMPORT COMMAND AS NEEDED TO MATCH THE TYPE OF DATA FILE
d$year=factor(d$year)
d$degree=factor(d$degree)
d$padeg=factor(d$padeg)
d$madeg=factor(d$madeg)
d$spdeg=factor(d$spdeg)
d$female=ifelse(d$sex=="female", 1, 0)
d$sex=NULL
d$race=factor(d$race)
d$id=1:dim(d)[1]

#impute the data
library(Amelia)
impdat=amelia(d, m=20, noms=c("degree", "padeg", "madeg", "spdeg", "race"), idvars=c("year", "wtss", "wtssnr", "id"), logs="realrinc")

#save in R format
setwd("~/Documents/Duke/Predictions with MI data/")
save(impdat, file="gss_imputed.rdata")

#export to stata format - all predictions for linear transformations performed in Stata
setwd("~/Documents/Duke/Predictions with MI data/") #ALTER THE PATHNAME AS NEEDED TO STORE WHERE YOU WANT IT ON YOUR COMPUTER
write.amelia(obj=impdat, file.stem="gss_imputed", format="dta", separate=F, orig.data=T, impvar="imp")

## Non-linear predictions - Simulation
#NOTE: these simulations can take a very long time to run
library(MASS); library(Amelia); library(mitools); library(boot)

runsim=function(n,m,sigma,s,perc.miss,nreps){
	cat("\n", "starting simulations", "\n")
	n.miss=n*perc.miss
	#keeping track of progress
	printnum=seq(0,nreps, by=10)
	
	#get family objects - used in calculating SE's
	y=rbinom(30, 1, .4); y2=rpois(30, 1); x=rnorm(30)
	mod1=glm(y ~ x, family=binomial)
	mod2=glm(y ~ x, family=binomial(link="probit"))
	mod3=glm(y2 ~ x, family=poisson)
	fam.logit=family(mod1)
	fam.probit=family(mod2)
	fam.poisson=family(mod3)
	
	#matrix to store results in
	resultmat.logit=matrix(NA, nrow=nreps, ncol=7)
	resultmat.probit=matrix(NA, nrow=nreps, ncol=7)
	resultmat.poisson=matrix(NA, nrow=nreps, ncol=7)
	colnames(resultmat.logit)=colnames(resultmat.probit)=colnames(resultmat.poisson)=c("pred.cp", "se.cp", "pred.pc", "se.pc", "pred.fmi", "pred.full", "se.full")
	for(j in 1:nreps){
		d=mvrnorm(n, mu=rep(0,nrow(sigma)), Sigma=sigma)
		d=as.data.frame(d)
		ystar1 = .5*d[,"x1"] + .5*d[,"x2"] + .5*d[,"x3"] + rlogis(n)
		ystar2 = .5*d[,"x1"] + .5*d[,"x2"] + .5*d[,"x3"] + rnorm(n)
		d$y = ifelse(ystar1 > 0, 1, 0)
		d$y2 = ifelse(ystar2 > 0, 1, 0)
		d$y3 = rpois(n, exp(.5*d[,"x1"] + .5*d[,"x2"] + .5*d[,"x3"]))
		
		get.full.preds=function(modtype){
			fullmod=switch(modtype, logit=glm(y ~ x1 + x2 + x3, data=d, family=binomial), probit=glm(y2 ~ x1 + x2 + x3, data=d, family=binomial(link="probit")), poisson=glm(y ~ x1 + x2 + x3, data=d, family=poisson))
			full.linpred=s%*%coef(fullmod)
			full.nonlinpred=switch(modtype, logit=inv.logit(full.linpred), probit=pnorm(full.linpred), poisson=exp(full.linpred))
			V=vcov(fullmod)
			fam=switch(modtype, logit=fam.logit, probit=fam.probit, poisson=fam.poisson)
			full.se=s%*%V%*%t(s)*fam$mu.eta(full.linpred)
			return(c(full.nonlinpred, full.se))
		} #end get.full.preds()
		
		logitpreds=get.full.preds("logit")
		probitpreds=get.full.preds("probit")
		poissonpreds=get.full.preds("poisson")

		#generate missingness completely at random, equal amount in each variable
		dmiss=d
		dmiss=as.data.frame(apply(dmiss, 2, function(x) {x[sample(1:dim(dmiss)[1], n.miss, replace=F)]=NA; return(x)}))
		miss.y=is.na(dmiss$y); miss.y2=is.na(dmiss$y2); miss.y3=is.na(dmiss$y3)
		impdat=amelia(dmiss, m=m, p2s=0)
		implist=impdat$imputations
		
		##calculate predictions using both methods
		get.preds=function(modtype){
			if(modtype=="logit") mods=lapply(implist, function(x) {glm(y ~ x1 + x2 + x3, data=x, family=binomial, subset=miss.y==F)})
			if(modtype=="probit") mods=lapply(implist, function(x) {glm(y2 ~ x1 + x2 + x3, data=x, family=binomial(link="probit"), subset=miss.y2==F)})
			if(modtype=="poisson") mods=lapply(implist, function(x) {glm(y3 ~ x1 + x2 + x3, data=x, family=poisson, subset=miss.y3==F)})
			mi.mod=MIcombine(mods)
			#combine then predict
			linpred=s%*%coef(mi.mod)
			lin.se=sqrt(s%*%vcov(mi.mod)%*%t(s))
			nonlinpred.cp=switch(modtype, logit=inv.logit(linpred), probit=pnorm(linpred), poisson=exp(linpred))
			fam=switch(modtype, logit=fam.logit, probit=fam.probit, poisson=fam.poisson)
			nonlinpred.se.cp=lin.se*fam$mu.eta(linpred)
			#predict then combine
			linpreds.pc=sapply(mods, function(x) {s%*%coef(x)})
			nonlinpred.pc=switch(modtype, logit=mean(inv.logit(linpreds.pc)), probit=mean(pnorm(linpreds.pc)), poisson=mean(exp(linpreds.pc)))
			Vw.pc=mean(sapply(mods, function(x) {(sqrt(s%*%vcov(x)%*%t(s))*fam$mu.eta(s%*%coef(x)))^2}))
			Vb.pc=switch(modtype, logit=var(inv.logit(linpreds.pc)), probit=var(pnorm(linpreds.pc)), poisson=var(exp(linpreds.pc)))
			nonlinpred.se.pc=sqrt(Vw.pc + Vb.pc + Vb.pc/m)
			#get FMI
			v=(m-1)*(1+Vw.pc/(Vb.pc+Vb.pc/m))^2
			fmi=(Vb.pc + Vb.pc/m + 2/(v+3))/(Vw.pc + Vb.pc + Vb.pc/m)
			#return results
			c(nonlinpred.cp, nonlinpred.se.cp, nonlinpred.pc, nonlinpred.se.pc, fmi)
		} #end get.preds()
		
		resultmat.logit[j,]=c(get.preds("logit"), logitpreds[1], logitpreds[2])
		resultmat.probit[j, ]=c(get.preds("probit"), probitpreds[1], probitpreds[2])
		resultmat.poisson[j, ]=c(get.preds("poisson"), poissonpreds[1], poissonpreds[2])	
		if(j %in% printnum) cat(j, "\n") else cat(".")
	} #end j loop
	return(list(results.logit=resultmat.logit, results.probit=resultmat.probit, results.poisson=resultmat.poisson))
} #end funciton runsim()

#function to extract the relevant results from the simulation run
extract.results=function(x, modtype, compare=c("trueval", "fullsampval")){
  compare=match.arg(compare)
  x=switch(modtype, logit=x$results.logit, probit=x$results.probit, poisson=x$results.poisson)
  if(compare=="trueval"){
    truepred=s%*%c(0,.5,.5,.5)
    truepred=switch(modtype, logit=inv.logit(truepred), probit=pnorm(truepred), poisson=exp(truepred))
    bias.cp=abs(truepred-mean(x[,"pred.cp"]))
    bias.pc=abs(truepred-mean(x[,"pred.pc"]))
    mse.cp=mean((truepred-x[,"pred.cp"])^2)
    mse.pc=mean((truepred-x[,"pred.pc"])^2)
  }
  if(compare=="fullsampval"){
    bias.cp=abs(mean(x[,"pred.full"])-mean(x[,"pred.cp"]))
    bias.pc=abs(mean(x[,"pred.full"])-mean(x[,"pred.pc"]))
    mse.cp=mean((x[,"pred.full"]-x[,"pred.cp"])^2)
    mse.pc=mean((x[,"pred.full"]-x[,"pred.pc"])^2)
  }
	out=c(bias.cp, bias.pc, mse.cp, mse.pc)
	names(out)=c("bias.cp", "bias.pc", "mse.cp", "mse.pc")
	return(out)
} #end extract.results()

#function takes a list of results in order logit, probit, poisson
plotsims=function(x, type=c("mse", "bias"), simtype=c("m", "n", "perc.miss")){
  type=match.arg(type)
  simtype=match.arg(simtype)
  xlabel=switch(simtype, m="Number of Imputations (m)", n="Sample Size", perc.miss="Percent of Data Missing")
  ylabel=switch(type, mse="Mean Squared Error", bias="Bias")
  whichrow.cp=switch(type, mse="mse.cp", bias="bias.cp")
  whichrow.pc=switch(type, mse="mse.pc", bias="bias.pc")
  modnames=c("Logit", "Probit", "Poisson")
  xvar=switch(simtype, m=mvals, n=nvals, perc.miss=missvals)
  par(mfrow=c(1,3))
  for(i in 1:length(x)){
    ylims=c(min(c(x[[i]][whichrow.cp,], x[[i]][whichrow.pc,])), max(c(x[[i]][whichrow.cp,], x[[i]][whichrow.pc,])))
    plot(x[[i]][whichrow.cp,] ~ xvar, type="l", main=modnames[[i]], xlab=xlabel, ylab=ylabel, ylim=ylims)
    lines(xvar, x[[i]][whichrow.pc,], lty=2)
    legend("topleft", legend=c("CP", "PC"), lty=c(1,2))
  } #end i loop
} #end plotsims()

#simulation parameters
s=matrix(c(1,1,1,1), nrow=1)
#covariance matrix of predictors
sigma=matrix(.5, nrow=3, ncol=3)
diag(sigma)=1
rownames(sigma)=colnames(sigma)=c("x1","x2","x3")

#run the simulation with realistic values, m increasing in step with percent missing
argslist=list()
mvals=seq(10,50,10)
missvals=seq(.1,.5,.1)
for(i in 1:length(mvals)){
	argslist[[i]]=list(n=250, m=mvals[[i]], sigma=sigma, s=s, perc.miss=missvals[[i]], nreps=1000)
}
sims.real25=lapply(argslist, function(x) do.call(runsim, x))
#NOTE: it is likely that this simulation will run into an error at higher levels of missing data due to the small sample size - if this happens, the simulation can be run in pieces, like the following
#sims.real25b=runsim(n=250, m=50, sigma=sigma, s=s, perc.miss=.5, nreps=250)
#the various simulation runs should then be combined into one set of results to match the following format
# list of length 5, each index is a different percent of data missing, .1 to .5
# each of these 5 lists is a list of length 3, with elements named "results.logit", "results.probit", results.poisson", 
# each of these lists contains the estimated values from each simulation run, for a total of 1000 rows (or however many repetitions you run) and 7 columns 

argslist=list()
mvals=seq(10,50,10)
missvals=seq(.1,.5,.1)
for(i in 1:length(mvals)){
	argslist[[i]]=list(n=500, m=mvals[[i]], sigma=sigma, s=s, perc.miss=missvals[[i]], nreps=1000)
}
sims.real50=lapply(argslist, function(x) do.call(runsim, x))

argslist=list()
mvals=seq(10,50,10)
missvals=seq(.1,.5,.1)
for(i in 1:length(mvals)){
	argslist[[i]]=list(n=1000, m=mvals[[i]], sigma=sigma, s=s, perc.miss=missvals[[i]], nreps=1000)
}
sims.real100=lapply(argslist, function(x) do.call(runsim, x))

#plot results
#compared to true value
m.results.logit25=sapply(sims.real25, extract.results, modtype="logit", compare="trueval")
m.results.probit25=sapply(sims.real25, extract.results, modtype="probit", compare="trueval")
m.results.poisson25=sapply(sims.real25, extract.results, modtype="poisson", compare="trueval")
colnames(m.results.logit25)=colnames(m.results.probit25)=colnames(m.results.poisson25)=missvals
plotsims(list(m.results.logit25, m.results.probit25, m.results.poisson25), "mse", "perc.miss")

m.results.logit50=sapply(sims.real50, extract.results, modtype="logit", compare="trueval")
m.results.probit50=sapply(sims.real50, extract.results, modtype="probit", compare="trueval")
m.results.poisson50=sapply(sims.real50, extract.results, modtype="poisson", compare="trueval")
colnames(m.results.logit50)=colnames(m.results.probit50)=colnames(m.results.poisson50)= missvals
plotsims(list(m.results.logit50, m.results.probit50, m.results.poisson50), "mse", "perc.miss")

m.results.logit100=sapply(sims.real100, extract.results, modtype="logit", compare="trueval")
m.results.probit100=sapply(sims.real100, extract.results, modtype="probit", compare="trueval")
m.results.poisson100=sapply(sims.real100, extract.results, modtype="poisson", compare="trueval")
colnames(m.results.logit100)=colnames(m.results.probit100)=colnames(m.results.poisson100)= missvals
plotsims(list(m.results.logit100, m.results.probit100, m.results.poisson100), "mse", "perc.miss")


##plot all on same graph
plotsims2=function(x, type=c("mse", "bias"), simtype=c("m", "n", "perc.miss")){
	type=match.arg(type)
  	simtype=match.arg(simtype)
  	xlabel=switch(simtype, m="Number of Imputations (m)", n="Sample Size", perc.miss="Percent of Data Missing")
  	ylabel=switch(type, mse="Mean Squared Error", bias="Bias")
  	whichrow.cp=switch(type, mse="mse.cp", bias="bias.cp")
  	whichrow.pc=switch(type, mse="mse.pc", bias="bias.pc")
  	modnames=c("Logit", "Probit", "Poisson")
  	xvar=switch(simtype, m=mvals, n=nvals, perc.miss=missvals)
  	  par(mfrow=c(1,3))
  for(i in 1:length(x)){
  	ylims=c(0, max(c(x[[i]][[1]][whichrow.cp,], x[[i]][[1]][whichrow.pc,])))
  	plot(x[[i]][[1]][whichrow.cp,] ~ xvar, type="l", main=modnames[[i]], xlab=xlabel, ylab=ylabel, ylim=ylims)
  	lines(xvar, x[[i]][[1]][whichrow.pc,], lty=2)
  	for(j in 2:length(x[[i]])){
  		lines(xvar, x[[i]][[j]][whichrow.cp,], lty=1)
    	lines(xvar, x[[i]][[j]][whichrow.pc,], lty=2)
  	}  
    legend("topleft", legend=c("CP", "PC"), lty=c(1,2))
  } #end i loop
}

plotlist=list(logit=list(m.results.logit25, m.results.logit50, m.results.logit100), probit=list(m.results.probit25, m.results.probit50, m.results.probit100), poisson=list(m.results.poisson25, m.results.poisson50, m.results.poisson100))
plotsims2(plotlist, "mse", "perc.miss")
